/** @odoo-module */

import { Order, Orderline } from "@point_of_sale/app/store/models";
import { roundDecimals as round_di } from "@web/core/utils/numbers";
import { patch } from "@web/core/utils/patch";
import { jsonrpc } from "@web/core/network/rpc_service"

patch(Order.prototype, {
    async fetchPriceARS(product) {
        try {
            const data = await jsonrpc("/web/dataset/call_kw/product.product/get_rate", {
                model: "product.product",
                method: "get_rate",
                args: [product.id],
                kwargs: {},
            });
            if (data && data.rate) {
                return data.rate;
            }
        } catch (error) {
            console.error("Error obteniendo:", error);
        }
    },
    async add_product(product, options) {
        this.pos.numpadMode = 'quantity';
        if (
            this.pos.doNotAllowRefundAndSales() &&
            this._isRefundOrder() &&
            (!options.quantity || options.quantity > 0)
        ) {
            this.pos.env.services.popup.add(ErrorPopup, {
                title: _t("Refund and Sales not allowed"),
                body: _t("It is not allowed to mix refunds and sales"),
            });
            return;
        }

        if (this._printed) {
            // when adding product with a barcode while being in receipt screen
            this.pos.removeOrder(this);
            return await this.pos.add_new_order().add_product(product, options);
        }

        this.assert_editable();
        options = options || {};
        const quantity = options.quantity ? options.quantity : 1;
        const line = new Orderline(
            { env: this.env },
            { pos: this.pos, order: this, product: product, quantity: quantity }
        );
        const productRate = await this.fetchPriceARS(line.product)
        
        line.product.lst_price = line.product.lst_price * productRate
        this.fix_tax_included_price(line);
        this.set_orderline_options(line, options);
        line.set_full_product_name();
        var to_merge_orderline;
        for (var i = 0; i < this.orderlines.length; i++) {
            if (this.orderlines.at(i).can_be_merged_with(line) && options.merge !== false) {
                to_merge_orderline = this.orderlines.at(i);
            }
        }

        if (to_merge_orderline) {
            to_merge_orderline.merge(line);
            this.select_orderline(to_merge_orderline);
        } else {
            this.add_orderline(line);
            this.select_orderline(this.get_last_orderline());
        }

        if (options.draftPackLotLines) {
            this.selected_orderline.setPackLotLines({
                ...options.draftPackLotLines,
                setQuantity: options.quantity === undefined,
            });
        }

        if (options.comboLines?.length) {
            await this.addComboLines(line, options);
            // Make sure the combo parent is selected.
            this.select_orderline(line);
        }

        this.hasJustAddedProduct = true;
        clearTimeout(this.productReminderTimeout);
        this.productReminderTimeout = setTimeout(() => {
            this.hasJustAddedProduct = false;
        }, 3000);

        return line;
    }
    
});

patch(Orderline.prototype, {
    getDisplayData() {
        return {
            ...super.getDisplayData(),
            defaultCode: this.product.default_code,
            stockAvailable: this.product.stock_available,
        };
    },
    set_unit_price(price) {
        this.order.assert_editable();
        const { numpadMode } = this.pos;
        const parsed_price = !isNaN(price)
            ? price
            : isNaN(parseFloat(price))
                ? 0
                : oParseFloat("" + price);

        const total_price = parsed_price || 0;
        if (numpadMode === "price") {
            const taxes = this.get_applicable_taxes().map((tax) => tax.amount / 100);
            const totalTaxes = taxes.reduce((acc, tax) => acc + tax, 0);
            this.price = round_di(total_price / (1 + totalTaxes) || 0, this.pos.dp["Product Price"]);
        } else {
            this.price = round_di(total_price, this.pos.dp["Product Price"]);
        }
    },
});
